@@ -28,6 +28,7 @@ History:
                     Added boolean member variable _enableCallbacks.
                     Added a guard to prevent infinite recursion while running callbacks.
 2010-06-07 ROwen    Added a few commented-out print statements.
+2010-08-26 ROwen    Tweaked commented-out print statements.
 """
 import re
 import sys
@@ -42,15 +43,14 @@ class _DisableCallbacksContext(object):
         self.callbackObj = callbackObj
     
     def __enter__(self):
-        temp = self.callbackObj._enableCallbacks
         self.initialCallbacksEnabled = self.callbackObj._enableCallbacks
         self.callbackObj._enableCallbacks = False
-#        print "%s.__enter__; _enableCallbacks was %s; is %s" % (self.callbackObj, temp, self.callbackObj._enableCallbacks)
+#        print "%s.__enter__; _enableCallbacks %s -> %s" % (self.callbackObj, self.initialCallbacksEnabled, self.callbackObj._enableCallbacks)
     
     def __exit__(self, type, value, traceback):
         temp = self.callbackObj._enableCallbacks
         self.callbackObj._enableCallbacks = self.initialCallbacksEnabled
-#        print "%s.__exit__; _enableCallbacks was %s; is %s" % (self.callbackObj, temp, self.callbackObj._enableCallbacks)
+#        print "%s.__exit__; _enableCallbacks %s -> %s" % (self.callbackObj, temp, self.callbackObj._enableCallbacks)
 
 
 class BaseMixin(object):
